library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity shift_counter is
port (
		clk_in		: in STD_LOGIC;
		rst_in		: in STD_LOGIC;
		count_in	: in STD_LOGIC;
		up_in		: in STD_LOGIC;
		q_out		: out STD_LOGIC_VECTOR(3 downto 0)
	);
end shift_counter;

architecture behavioral of shift_counter is
	signal	q 			: STD_LOGIC_VECTOR(3 downto 0);
	signal	sig_up		: STD_LOGIC;
	signal	sig_down	: STD_LOGIC;

	signal 	q_up		: STD_LOGIC_VECTOR(3 downto 0);
	signal	q_down		: STD_LOGIC_VECTOR(3 downto 0); 
	signal	q_left		: STD_LOGIC_VECTOR(3 downto 0); 
	signal	q_right		: STD_LOGIC_VECTOR(3 downto 0); 
begin
	q_out 		<= q;
	sig_up		<= q(3) AND (q(0) OR q(1) OR q(2));
	sig_down	<= (q(3) AND (q(1) OR q(2))) OR 
				   ((NOT q(0)) AND (NOT q(1)) AND (NOT q(2)) AND (NOT q(3)));

	-- enacba za stetje navzgor: q(i) <= q(i) XOR (q(i-1) AND ... AND q(0))
	-- enacba za stetje navzdol: q(i) <= q(i) XOR (NOT q(i-1) AND ... AND NOT q(0))

	q_up(0)		<= NOT sig_up AND NOT q(0);
	q_up(1)		<= NOT sig_up AND (q(1) XOR q(0));
	q_up(2)		<= NOT sig_up AND (q(2) XOR (q(1) AND q(0)));
	q_up(3)		<= NOT sig_up AND (q(3) XOR (q(2) AND q(1) AND q(0)));

	q_down(0)	<= sig_down OR (NOT sig_down AND NOT q(0));
	q_down(1) 	<= NOT sig_down AND (q(1) XOR NOT q(0));
	q_down(2)	<= NOT sig_down AND (q(2) XOR (NOT q(1) AND NOT q(0)));
	q_down(3)	<= sig_down OR (NOT sig_down AND (q(3) XOR (NOT q(2) AND NOT q(1) AND NOT q(0))));

	q_left		<= q(2 downto 0) & q(3);
	q_right		<= q(0) & q(3 downto 1);

	process (clk_in)
	begin
		if (rising_edge(clk_in)) then
		
		
			q(0)	<= NOT rst_in AND (
					(		count_in AND 	 up_in AND q_up(0)		) OR
					(		count_in AND NOT up_in AND q_down(0)	) OR
					(	NOT count_in AND	 up_in AND q_right(0)	) OR 
					(	NOT count_in AND NOT up_in AND q_right(0)	)
				);

			q(1)	<= NOT rst_in AND (
					(		count_in AND 	 up_in AND q_up(1)		) OR
					(		count_in AND NOT up_in AND q_down(1)	) OR
					(	NOT count_in AND	 up_in AND q_right(1)	) OR 
					(	NOT count_in AND NOT up_in AND q_right(1)	)
				);

			q(2)	<= NOT rst_in AND (
					(		count_in AND 	 up_in AND q_up(2)		) OR
					(		count_in AND NOT up_in AND q_down(2)	) OR
					(	NOT count_in AND	 up_in AND q_right(2)	) OR 
					(	NOT count_in AND NOT up_in AND q_right(2)	)
				);

			q(3)	<= NOT rst_in AND (
					(		count_in AND 	 up_in AND q_up(3)		) OR
					(		count_in AND NOT up_in AND q_down(3)	) OR
					(	NOT count_in AND	 up_in AND q_right(3)	) OR 
					(	NOT count_in AND NOT up_in AND q_right(3)	)
				);
				
		end if;
	end process;

end;

